HITFACTORPY_DB_SQLALCHEMY_URL = "HITFACTORPY_DB_SQLALCHEMY_URL"
HITFACTORPY_DB_ALEMBIC_DIR = "HITFACTORPY_DB_ALEMBIC_DIR"
HITFACTORPY_DB_USERNAME = "HITFACTORPY_DB_USERNAME"
HITFACTORPY_DB_PASSWORD = "HITFACTORPY_DB_PASSWORD"
HITFACTORPY_DB_HOST = "HITFACTORPY_DB_HOST"
HITFACTORPY_DB_PORT = "HITFACTORPY_DB_PORT"
HITFACTORPY_DB_DATABASE_NAME = "HITFACTORPY_DB_DATABASE_NAME"
HITFACTORPY_DB_CONNECTION_SCHEME = "HITFACTORPY_DB_CONNECTION_SCHEME"
