"""add stage number

Revision ID: c27a53ccd1d3
Revises: e3f53a281f1a
Create Date: 2023-01-23 22:08:32.065469

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "c27a53ccd1d3"
down_revision = "e3f53a281f1a"
branch_labels = None
depends_on = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("match_report_stage", sa.Column("stage_number", sa.Integer()))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("match_report_stage", "stage_number")
    # ### end Alembic commands ###
